#!/bin/sh

DEFAULT_INSTALL_DIR=/opt/appoptics
INSTALL_DIR=${DEFAULT_INSTALL_DIR}
APPOPTICS_CONF="${INSTALL_DIR}/etc/config.yaml"
SNAP_SERVICE_NAME=appoptics-snapteld
AGENT_DISPLAY_NAME="AppOptics Host Agent"
SNAP_TMP_DIR=/tmp/appoptics-snaptel
SNAP_LOG_DIR=/var/log/appoptics
PID_DIR=/var/run/appoptics
SYMLINK_DIR=/usr/local/bin
LOG_ROTATION_CONF_DIR=/etc/logrotate.d

SYSTEMCTL_BIN=`command -v systemctl 2>/dev/null`
CHKCONFIG_BIN=`command -v chkconfig 2>/dev/null`
SERVICE_BIN=`command -v service 2>/dev/null`
RM_CMD=/bin/rm

ROOT_DIR=`pwd`

# Settings for the user used to run the daemon
APPOPTICS_USERNAME=appoptics
APPOPTICS_GROUPNAME=appoptics

# truth values
__IS_TRUE=1
__IS_FALSE=0

# Contains the current installation step of the post installer
__step=1

_ECHO_DEBUG=${_ECHO_DEBUG:-${__IS_FALSE}}
_SD_NO_COLOR=${_SD_NO_COLOR:-${__IS_TRUE}}

__increment_step()
{
   __step=$((__step+1))
}

_COLORS=${BS_COLORS:-$(tput colors 2>/dev/null || echo 0)}
detect_color_support() 
{
    if [ $? -eq 0 ] && [ "$_COLORS" -gt 2 ] && [ $_SD_NO_COLOR -eq $__IS_FALSE ] ; then
        RC="\033[1;31m"
        GC="\033[1;32m"
        BC="\033[1;34m"
        YC="\033[1;33m"
        EC="\033[0m"
    else
        RC=""
        GC=""
        BC=""
        YC=""
        EC=""
    fi
}
detect_color_support 

# Override built-in echo for consistency. Has newline at end
echo () (
fmt=%s end=\\n IFS=" "

while [ $# -gt 1 ] ; do
    case "$1" in
        [!-]*|-*[!ne]*) break ;;
        *ne*|*en*) fmt=%b end= ;;
        *n*) end= ;;
        *e*) fmt=%b ;;
    esac
    shift
done

printf "$fmt$end" "$*"
)

echored() {
    printf "${RC}%s${EC}\n" "$*";
}

echogreen() {
    printf "${GC}%s${EC}\n" "$*";
}

echoblue() {
    printf "${BC}%s${EC}\n" "$*";
}

echoyellow() {
    printf "${YC}%s${EC}\n" "$*";
}

exists()
{
  command -v "$1" >/dev/null 2>&1
}

now()
{
  echo $(date +'%Y-%m-%d-%H_%M_%S')
}

echoerror() {
    printf "${RC} * ERROR${EC}: %s\n" "$*" 1>&2;
}

info ()
{
    printf "${GC} *  INFO${EC}: %s\n" "$*";
}

warning ()
{
  printf "${YC} *  WARN${EC}: %s\n" "$*";
}

echodebug()
{
    if [ "$_ECHO_DEBUG" = "$__IS_TRUE" ]; then
        printf "${BC} * DEBUG${EC}: %s\n" "$*";
    fi
}

error ()
{
  error_code=${1}
  error_description=${2:-""}
  error_date_time="`echo $(date +'%Y-%m-%d at %H:%M:%S')`"
  echoerror "######################################################"
  echoerror "An error has occured during uninstallation of the agent."
  echoerror
  echoerror "Description: ${error_description}"
  echoerror "Error code : ${error_code}"
  echoerror "When       : ${error_date_time}"
  echoerror "######################################################"
}

remove_install_directories()
{
    echodebug "Deleting support directories ..."
    ${RM_CMD} -rf ${SNAP_TMP_DIR}
    ${RM_CMD} -rf ${SNAP_LOG_DIR}
    ${RM_CMD} -rf ${PID_DIR}
    echodebug "directories removed."
}

remove_symlinks()
{
    echodebug "Deleting symlinks ..."
    ${RM_CMD} -f ${SYMLINK_DIR}/appoptics-plugin
    ${RM_CMD} -f ${SYMLINK_DIR}/snaptel
    ${RM_CMD} -f ${LOG_ROTATION_CONF_DIR}/appoptics-snaptel
    echodebug "symlinks removed."
}

remove_empty_directories()
{
    echodebug "Deleting empty directories ..."

    # On RedHat platforms, the following directories are left empty after removing the package.
    # Remove the empty directories to have a consistent behaviour cross-platforms.
    ${RM_CMD} -rf ${INSTALL_DIR}/autoload
    ${RM_CMD} -rf ${INSTALL_DIR}/bin
    ${RM_CMD} -rf ${INSTALL_DIR}/sbin
    ${RM_CMD} -rf ${INSTALL_DIR}/support

    echodebug "directories removed."
}

remove_service()
{
    if [ -x "${SYSTEMCTL_BIN}" ]; then
        # Remove service
        echodebug "Remove configuration for ${AGENT_DISPLAY_NAME} service ..."
        ${RM_CMD} -f /lib/systemd/system/${SNAP_SERVICE_NAME}.service
    fi
}

main()
{
    remove_service
    remove_symlinks
    remove_install_directories
    remove_empty_directories
}

main

