#!/bin/sh

DEFAULT_INSTALL_DIR=/opt/appoptics
INSTALL_DIR=${DEFAULT_INSTALL_DIR}
APPOPTICS_CONF="${INSTALL_DIR}/etc/config.yaml"
SNAP_SERVICE_NAME=appoptics-snapteld
AGENT_DISPLAY_NAME="AppOptics Host Agent"
SNAP_TMP_DIR=/tmp/appoptics-snaptel
SNAP_LOG_DIR=/var/log/appoptics
PID_DIR=/var/run/appoptics
SYMLINK_DIR=/usr/local/bin
DAEMON_NAME=snapteld
UPGRADE_SAVE_TOKEN_FILE=/opt/appoptics/etc/upgrade.token.backup
RM_CMD=/bin/rm

SYSTEMCTL_BIN=`command -v systemctl 2>/dev/null`
CHKCONFIG_BIN=`command -v chkconfig 2>/dev/null`
SERVICE_BIN=`command -v service 2>/dev/null`
DPKG_QUERY_BIN=`command -v dpkg-query 2>/dev/null`

# Settings for the user used to run the daemon
APPOPTICS_USERNAME=appoptics
APPOPTICS_GROUPNAME=appoptics

# truth values
__IS_TRUE=1
__IS_FALSE=0

_ECHO_DEBUG=${_ECHO_DEBUG:-${__IS_FALSE}}
_SD_NO_COLOR=${_SD_NO_COLOR:-${__IS_TRUE}}

# Contains the current installation step of the post installer
__step=1


# Return error codes
readonly SUCCESS=0
readonly FAILURE=1
readonly FAILURE_SERVICE_NOT_RUNNING=2
readonly FAILURE_NOT_ENOUGH_RIGHTS=3
readonly FAILURE_UNABLE_TO_EXTRACT=4
readonly FAILURE_MISSING_EXECUTABLE=5

__increment_step()
{
   __step=$((__step+1))
}

_COLORS=${BS_COLORS:-$(tput colors 2>/dev/null || echo 0)}
detect_color_support() 
{
    if [ $? -eq 0 ] && [ "$_COLORS" -gt 2 ] && [ $_SD_NO_COLOR -eq $__IS_FALSE ] ; then
        RC="\033[1;31m"
        GC="\033[1;32m"
        BC="\033[1;34m"
        YC="\033[1;33m"
        EC="\033[0m"
    else
        RC=""
        GC=""
        BC=""
        YC=""
        EC=""
    fi
}
detect_color_support 

# Override built-in echo for consistency. Has newline at end
echo () (
fmt=%s end=\\n IFS=" "

while [ $# -gt 1 ] ; do
    case "$1" in
        [!-]*|-*[!ne]*) break ;;
        *ne*|*en*) fmt=%b end= ;;
        *n*) end= ;;
        *e*) fmt=%b ;;
    esac
    shift
done

printf "$fmt$end" "$*"
)

echored() {
    printf "${RC}%s${EC}\n" "$*";
}

echogreen() {
    printf "${GC}%s${EC}\n" "$*";
}

echoblue() {
    printf "${BC}%s${EC}\n" "$*";
}

echoyellow() {
    printf "${YC}%s${EC}\n" "$*";
}

exists()
{
  command -v "$1" >/dev/null 2>&1
}

now()
{
  echo $(date +'%Y-%m-%d-%H_%M_%S')
}

echoerror() {
    printf "${RC} * ERROR${EC}: %s\n" "$*" 1>&2;
}

info ()
{
    printf "${GC} *  INFO${EC}: %s\n" "$*";
}

warning ()
{
  printf "${YC} *  WARN${EC}: %s\n" "$*";
}

echodebug()
{
    if [ "$_ECHO_DEBUG" = "$__IS_TRUE" ]; then
        printf "${BC} * DEBUG${EC}: %s\n" "$*";
    fi
}

error ()
{
  error_code=${1}
  error_description=${2:-""}
  error_date_time="`echo $(date +'%Y-%m-%d at %H:%M:%S')`"
  echoerror "######################################################"
  echoerror "An error has occurred during upgrade of the agent."
  echoerror
  echoerror "Description: ${error_description}"
  echoerror "Error code : ${error_code}"
  echoerror "When       : ${error_date_time}"
  echoerror "######################################################"
}

failed ()
{
  error_code=${1}
  echored "###################################################"
  echored "An error occurred during the installation of ${AGENT_DISPLAY_NAME}."
  echored "Please contact AppOptics customer service for support at support@appoptics.com."
  echored "###################################################"
}

is_process_running() 
{
    ps -C ${1} > /dev/null 2>&1
}

daemon_running()
{
   is_process_running ${DAEMON_NAME}
}

any_plugins_running()
{
   pgrep -l 'snap-plugin' > /dev/null 2>&1
}

kill_agent()
{
    if daemon_running; then
        echodebug "Killing ${AGENT_DISPLAY_NAME} service ..."
        pkill -TERM -U ${APPOPTICS_USERNAME} ${DAEMON_NAME}
        sleep 1
        pkill -KILL -U ${APPOPTICS_USERNAME} ${DAEMON_NAME}
    fi

    if any_plugins_running; then
        echodebug "Killing all processes running under ${APPOPTICS_USERNAME} ..."
        pkill -KILL -U ${APPOPTICS_USERNAME}
    fi
}

stop_service()
{
    echodebug "Stopping ${AGENT_DISPLAY_NAME} service ..."
    if [ -x "${SYSTEMCTL_BIN}" ]; then
        # Stop service
        ${SYSTEMCTL_BIN} stop ${SNAP_SERVICE_NAME}
    else
        # Stop service
        ${SERVICE_BIN} ${SNAP_SERVICE_NAME} stop
    fi

    kill_agent
}

check_previous_config()
{
    if [ -e ${APPOPTICS_CONF} ] ; then
        info "Found previous ${APPOPTICS_CONF} configuration file."

        # Create backup copy of the configuration file in case gets overritten
        config_filename=$(basename "${APPOPTICS_CONF}")
        config_backup_filename="${HOME}/${config_filename}.backup.appoptics-snaptel.`now`"
        info "Create backup of ${APPOPTICS_CONF} configuration file to ${config_backup_filename}."
        cp ${APPOPTICS_CONF} ${config_backup_filename}
        chmod 640 ${config_backup_filename}

        # Save token before starting the upgrade which might overwrite the configuration file
        old_token=$(grep token ${APPOPTICS_CONF} | awk '{gsub(/"/, "", $2); print $2}')

        # Check if tokne has been set during upgrade
        if [ -n "${APPOPTICS_TOKEN}" ] ; then
            echodebug "Found APPOPTICS_TOKEN enviroment variable set to ${APPOPTICS_TOKEN}."

            # Check if current token is different than the one saved in configuration file, warn user if different, and use the new token provided
            if [ "${old_token}" != "${APPOPTICS_TOKEN}" ] ; then
                warning "${AGENT_DISPLAY_NAME} was previously installed with token ${old_token} which is different than ${APPOPTICS_TOKEN} provided with this installation."
                warning "Upgrade will update ${APPOPTICS_CONF} configuration file with the provided ${APPOPTICS_TOKEN} token."
                old_token=${APPOPTICS_TOKEN}
            fi
        fi
        echo ${old_token} > ${UPGRADE_SAVE_TOKEN_FILE}
        chmod 640 ${UPGRADE_SAVE_TOKEN_FILE}
    fi
}

remove_unused_symlinks()
{
    echodebug "Deleting unused symlinks ..."
    ${RM_CMD} -f ${SYMLINK_DIR}/appoptics-plugin
    echodebug "symlinks removed."
}

main()
{
    check_previous_config
    stop_service
    remove_unused_symlinks
    exit ${SUCCESS}
}

main


